@@ -1,6 +1,6 @@
 #include "keys.h"
 
-
+extern QString keys_path;
 
 void sh_key::keyPressEvent ( QKeyEvent * event ){
     this->setText(QString("%1").arg(event->nativeScanCode()));
@@ -16,10 +16,16 @@ sh_key::sh_key(QWidget *parent) :
 
 
 
-keys::keys(QWidget *parent) :
+keys::keys(QWidget *parent, QString filename, QString name) :
         QWidget(parent)
 {
+    if (filename.isEmpty())
+        filename = defaultFile();
+    m_Filename = filename;
 
+    if (name.isEmpty())
+        name = "Default";
+    m_Name = name;
     ks=QStringList()<<tr("Left")<<tr("Right")<<tr("Up")
                                 <<tr("Down")<<tr("o")<<tr("x")<<tr("triangle")
                                 <<tr("square")<<tr("Select")<<tr("Start")
@@ -52,9 +58,9 @@ keys::keys(QWidget *parent) :
     grid->setVerticalSpacing(10);
 
     // load current
-    QFile file("/opt/psx4m/keys");
+    QFile file(m_Filename);
     if (!file.open(QIODevice::ReadOnly | QIODevice::Text)){
-        QMessageBox msg;
+        QMessageBox msg(this);
         msg.setText(tr("Couldn't open previous key config"));
         msg.exec();
     }
@@ -85,20 +91,12 @@ keys::keys(QWidget *parent) :
 void keys::hideEvent(QHideEvent *)
 {
     if (!this->isVisible()){
-        QDialog *dlg=new QDialog(this);
-        dlg->setWindowTitle(tr("Confirm"));
-        QGridLayout *lay=new QGridLayout(dlg);
-        lay->setSizeConstraint(QLayout::SetMinAndMaxSize);
-        QLabel *lbl=new QLabel(tr("Do you want to save settings?"),dlg);
-        lay->addWidget(lbl,1,0,1,2);
-        QPushButton *yes=new QPushButton(tr("Yes"),dlg);
-        QPushButton *no=new QPushButton(tr("No"),dlg);
-        connect (yes,SIGNAL(clicked()),dlg,SLOT(accept()));
-        connect (no,SIGNAL(clicked()),dlg,SLOT(reject()));
-        connect (dlg,SIGNAL(accepted()),this,SLOT(save()));
-        lay->addWidget(yes,2,0);
-        lay->addWidget(no ,2,1);
-        dlg->exec();
+        QMessageBox msg(this);
+        msg.setWindowTitle(tr("Confirm"));
+        msg.setText(tr("Do you want to save settings?"));
+        msg.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
+        if (msg.exec() == QMessageBox::Yes)
+            save();
     }
 }
 
@@ -108,14 +106,35 @@ void keys::save()
     for (int i=0;i<ks.count();i++)
         sv=sv+QString("%1\n").arg(nkey[i]->text());
 
-    QFile file("/opt/psx4m/keys");
+    qDebug() << "Saving keys to" << m_Filename;
+    QFile file(m_Filename);
     if (!file.open(QIODevice::WriteOnly | QIODevice::Text)){
-        QMessageBox msg;
+        QMessageBox msg(this);
         msg.setText("Somethings goes wrong, check permission to keys file");
         msg.exec();
         return;
     }
 
+    QHash<QString, QString> list = configList();
+    if (!list.keys().contains(m_Name)){
+        list[m_Name] = m_Filename;
+    }
+
+    QSettings settings;
+    settings.beginWriteArray("keysconfig");
+    int index = 0;
+
+    QList<QString> keys = list.keys();
+    qSort(keys.begin(), keys.end());
+    foreach (QString name, keys){
+        settings.setArrayIndex(index);
+        settings.setValue("name", name);
+        settings.setValue("file", list[name]);
+        index++;
+    }
+    settings.endArray();
+    settings.sync();
+
     QTextStream out(&file);
     out <<sv;
 }
@@ -124,3 +143,40 @@ keys::~keys()
 {
 
 }
+
+int keys::keysConfigCount()
+{
+    QSettings* settings=new QSettings();
+
+    int count = settings->beginReadArray("keysconfig");
+    settings->endArray();
+    delete settings;
+
+    return count;
+}
+
+QString keys::defaultFile()
+{
+    return QString("%1keys").arg(keys_path);
+}
+
+QHash<QString, QString> keys::configList()
+{
+    QHash<QString, QString> res;
+    QSettings* settings = new QSettings();
+
+    int count = settings->beginReadArray("keysconfig");
+    for (int i=0; i<count; i++){
+        settings->setArrayIndex(i);
+        QString name = settings->value("name").toString();
+        QString file = settings->value("file").toString();
+        res[name] = file;
+    }
+    settings->endArray();
+    delete settings;
+
+    if (res.count() == 0 && QFile::exists(defaultFile()))
+        res["Default"] = defaultFile();
+
+    return res;
+}
