@@ -16,7 +16,7 @@ public:
     explicit sh_key(QWidget *parent = 0);
 
 protected:
-void keyPressEvent ( QKeyEvent * event ) ;
+    void keyPressEvent ( QKeyEvent * event ) ;
 
 
 };
@@ -27,13 +27,21 @@ class keys : public QWidget
     Q_OBJECT
 
 public:
-    explicit keys(QWidget *parent = 0);
+    explicit keys(QWidget *parent = 0, QString filename="", QString name="");
     ~keys();
+    static int keysConfigCount();
+    static QString defaultFile();
+    static QHash<QString, QString> configList();
+
 private:
     sh_key *nkey[24];
     QStringList ks;
+    QString m_Filename;
+    QString m_Name;
+
 private slots:
     void save();
+
 protected:
     void hideEvent(QHideEvent *);
 
