@@ -2,12 +2,15 @@
 #include "ui_mainwindow.h"
 #include "memorycardmanager.h"
 #include "savestateselect.h"
+#include "keysconfigselect.h"
 
 QString game;
 #ifdef Q_WS_MAEMO_5
-QString cfg_path="/home/user/.config/Team16/psx4m/";
+QString cfg_path ="/home/user/.config/Team16/psx4m/";
+QString keys_path="/home/user/.config/Team16/keys/";
 #else
-QString cfg_path= QString("%1/.config/Team16/psx4m/").arg(QDir::homePath());
+QString cfg_path  = QString("%1/.config/Team16/psx4m/").arg(QDir::homePath());
+QString keys_path = QString("%1/keys/").arg(cfg_path);
 #endif
 QString bios_path="/home/user/MyDocs/";
 QString rom_path="/home/user/MyDocs";
@@ -20,6 +23,13 @@ MainWindow::MainWindow(QWidget *parent) :
 {
     ui->setupUi(this);
 
+#ifdef Q_WS_MAEMO_5
+    //Copy defaul keys config if does not exist:
+    if (!QFile::exists(keys::defaultFile())){
+        QFile::copy("/opt/psx4m/keys", keys::defaultFile());
+    }
+#endif
+
     myProcess = NULL;
     settings=new QSettings();
     bios_path=settings->value("bios_path","/home/user/MyDocs/").toString();
@@ -38,12 +48,18 @@ MainWindow::MainWindow(QWidget *parent) :
 
     QDir cfg_dir(cfg_path);
     if (!cfg_dir.exists()){
-        if (!cfg_dir.mkdir(cfg_path))
+        if (!cfg_dir.mkpath(cfg_path))
             qDebug("Can't create cfg dir, permission error?");
         else
             cfg->save_cfg();
     }
 
+    QDir keys_dir(keys_path);
+    if (!keys_dir.exists()){
+        if (!keys_dir.mkpath(keys_path))
+            qDebug("Can't create keys dir, permission error?");
+    }
+
     ui->listWidget->setStyleSheet("QListView::item{ height: 86px; }");
     ui->listWidget->setAutoFillBackground(true);
     ui->listWidget->setIconSize(QSize(144,80));
@@ -60,7 +76,11 @@ MainWindow::MainWindow(QWidget *parent) :
     this->menuBar()->addAction("Edit Memory cards",this,SLOT(memory_cards()));
     this->menuBar()->addAction("Help",this,SLOT(open_help()));
 
-    connect(ui->listWidget,SIGNAL(currentRowChanged(int)),this,SLOT(currentRowChanged(int)));
+#ifdef Q_WS_MAEMO_5
+    connect(ui->listWidget, SIGNAL(itemActivated(QListWidgetItem*)), this, SLOT(rom_selected(QListWidgetItem*)));
+#else
+    connect(ui->listWidget, SIGNAL(itemClicked(QListWidgetItem*)), this, SLOT(rom_selected(QListWidgetItem*)));
+#endif
 }
 
 void MainWindow::get_roms()
@@ -92,15 +112,16 @@ void MainWindow::get_roms()
                 icon = QIcon(QPixmap::fromImage(image));
             }
         }
-        ui->listWidget->addItem(new QListWidgetItem(icon, fileInfo.fileName()));
-
         rom->FilePath = fileInfo.absoluteFilePath();
         rom->FileName = fileInfo.fileName();
+        rom->DisplayName = fileInfo.baseName().replace("_", " ");
         m_Roms.append(rom);
+
+        ui->listWidget->addItem(new QListWidgetItem(icon, rom->DisplayName));
     }
     if (m_Roms.count() > 0){
         ui->listWidget->setCurrentRow(0);
-        currentRowChanged(0);
+        rom_selected(NULL);
     }
 }
 
@@ -109,19 +130,27 @@ void MainWindow::rom_exec_savestate()
     SavestateSelect* dlg = new SavestateSelect(this, m_Roms.at(ui->listWidget->currentRow()));
     if (dlg->exec() == QDialog::Accepted){
         int slot = dlg->Selected();
-        rom_exec(slot);
-    }
+        int disc = dlg->SelectedDisc();
+        delete dlg;
 
+        if (slot >= 0){
+            qDebug() << "Savestate" << slot << "disc" << disc;
+            rom_exec(disc, slot);
+        }
+    }else
+        delete dlg;
+    rom_selected(NULL);
 }
 
 void MainWindow::rom_exec(){
-    rom_exec(-1);
+    rom_exec(0, -1);
 }
 
-void MainWindow::rom_exec(int slot)
+void MainWindow::rom_exec(int disc, int slot)
 {
     if (ui->listWidget->currentRow()!=-1){
-        game=ui->listWidget->currentItem()->text();
+        RomInfo* rom = m_Roms.at(ui->listWidget->currentRow());
+        game = rom->FileName;
         cfg->load_cfg();
 
         if (myProcess == NULL){
@@ -133,7 +162,9 @@ void MainWindow::rom_exec(int slot)
             connect(myProcess, SIGNAL(finished(int)), this, SLOT(process_finished(int)));
 
             QStringList args = current_config;
-            args << m_Roms.at(ui->listWidget->currentRow())->FilePath;
+            args << rom->FilePath;
+            if (disc > 0)
+                args << "-disc" << QString::number(disc);
             if (slot >= 0)
                 args << "-load" << QString::number(slot);
 
@@ -147,8 +178,16 @@ void MainWindow::rom_exec(int slot)
     }
 }
 
-void MainWindow::process_finished(int)
+void MainWindow::process_finished(int exitCode)
 {
+    if (exitCode){
+        QString error = myProcess->readAllStandardError();
+        QMessageBox msg(this);
+        msg.setWindowTitle(tr("Error"));
+        msg.setText(error);
+        msg.exec();
+    }
+
     ui->pushButton->setEnabled(true);
     ui->pushButton_2->setEnabled(true);
     ui->pbnSstate->setEnabled(true);
@@ -156,29 +195,30 @@ void MainWindow::process_finished(int)
     myProcess = NULL;
 }
 
-void MainWindow::currentRowChanged ( int  ){
-    if (ui->listWidget->currentRow()!=-1){
+void MainWindow::rom_selected(QListWidgetItem*){
+    if (myProcess == NULL && ui->listWidget->currentRow()!=-1){
+        RomInfo* rom = m_Roms.at(ui->listWidget->currentRow());
         game="";
         cfg->load_cfg();
-        game=ui->listWidget->currentItem()->text();
+        game = rom->FileName;
         cfg->load_cfg();
 
-        RomInfo* rom = m_Roms.at(ui->listWidget->currentRow());
-        if (rom->SaveStateName.isEmpty()){
+        if (rom->SaveStateNames.count() == 0){
             if (current_core == "pcsx"){
                 QProcess* p = new QProcess(this);
                 p->setWorkingDirectory("/home/user/");
                 QStringList args = QStringList() << "-cdfile" << rom->FilePath << "-get_sstatename" << "0";
                 p->start("/opt/maemo/usr/games/pcsx", args);
                 if (p->waitForFinished(3000)){
-                    QString out = p->readAll();
-                    QStringList lines = out.split("\n", QString::SkipEmptyParts);
-                    foreach (QString line, lines){
-                        line = line.trimmed();
-                        if (line.startsWith("SAVESTATE:")){
-                            QFileInfo f(line.remove(0, 8).trimmed());
-                            rom->SaveStateName = f.fileName();
-                            break;
+                    if (p->exitCode() == 0){
+                        QString out = p->readAll();
+                        QStringList lines = out.split("\n", QString::SkipEmptyParts);
+                        foreach (QString line, lines){
+                            line = line.trimmed();
+                            if (line.startsWith("SAVESTATE:")){
+                                QFileInfo f(line.remove(0, 8).trimmed());
+                                rom->SaveStateNames.append(f.fileName());
+                            }
                         }
                     }
                 }else{
@@ -187,11 +227,11 @@ void MainWindow::currentRowChanged ( int  ){
                 delete p;
             }else{
                 QFileInfo f(rom->FileName);
-                rom->SaveStateName = QString("%1.save.0").arg(f.fileName());
+                rom->SaveStateNames.append(QString("%1.save.0").arg(f.fileName()));
             }
         }
 
-        int count = count_save_states(m_Roms.at(ui->listWidget->currentRow())->SaveStateName);
+        int count = count_save_states(rom->SaveStateNames);
         ui->pbnSstate->setEnabled(count > 0);
 
 /*        QString scr = QString("%1/%2_bkg.png").arg(rom_path).arg(game);
@@ -236,9 +276,23 @@ void MainWindow::currentRowChanged ( int  ){
 }
 
 void MainWindow::set_keys(){
-    keyopt=new keys(this);
-    keyopt->show();
-    keyopt = NULL;
+
+    int count = keys::keysConfigCount();
+    if (count == 0){
+        keyopt=new keys(this);
+        keyopt->setWindowTitle(tr("Default config"));
+        keyopt->show();
+        keyopt = NULL;
+    }else{
+        KeysConfigSelect* dlg = new KeysConfigSelect(this);
+        if (dlg->exec() == QDialog::Accepted){
+            keyopt=new keys(this, dlg->Selected(), dlg->SelectedName());
+            keyopt->setWindowTitle(dlg->SelectedName());
+            keyopt->show();
+            keyopt = NULL;
+        }
+        delete dlg;
+    }
 }
 
 void MainWindow::def_cfg()
@@ -281,7 +335,7 @@ void MainWindow::set_rom_dir()
         settings->setValue("rom_path",rom_path);
         settings->sync();
         if (settings->status() != QSettings::NoError){
-            QMessageBox msg;
+            QMessageBox msg(this);
             msg.setText("Can't save gui config, permission error?");
             msg.exec();
         }
@@ -299,7 +353,7 @@ void MainWindow::set_bios_dir()
         settings->setValue("bios_path",bios_path);
         settings->sync();
         if (settings->status() != QSettings::NoError){
-            QMessageBox msg;
+            QMessageBox msg(this);
             msg.setText("Can't save gui config, permission error?");
             msg.exec();
         }
@@ -307,7 +361,7 @@ void MainWindow::set_bios_dir()
     }
 }
 
-void MainWindow::open_help() // Открытие страницы FAQ
+void MainWindow::open_help()
 {
     QDesktopServices::openUrl(QUrl("http://maemo.team16.ru/index.php?page=7"));
 }
@@ -321,15 +375,15 @@ MainWindow::~MainWindow()
 void MainWindow::rom_config()
 {
     if (ui->listWidget->currentRow()!=-1){
+        RomInfo* rom = m_Roms.at(ui->listWidget->currentRow());
         QString core = current_core;
-        game=ui->listWidget->currentItem()->text();
+        game = rom->FileName;
         cfg->load_cfg();
         cfg->setWindowTitle(tr("Config: %1").arg(game));
         cfg->show();
         if (core != current_core){
-            RomInfo* rom = m_Roms.at(ui->listWidget->currentRow());
-            rom->SaveStateName.clear();
-            currentRowChanged(ui->listWidget->currentRow());
+            rom->SaveStateNames.clear();
+            rom_selected(NULL);
         }
     }
 }
@@ -341,22 +395,26 @@ void MainWindow::memory_cards()
     dlg = NULL;
 }
 
-int MainWindow::count_save_states(QString name)
+int MainWindow::count_save_states(QStringList names)
 {
     int res = 0;
 
-    QFileInfo f(name);
-    QDir dir;
-    if (current_core == "pcsx")
-        dir.setPath(sstate_path);
-    else
-        dir.setPath(rom_path);
-    dir.setFilter(QDir::Files);
-    dir.setSorting(QDir::Name | QDir::IgnoreCase);
-    dir.setNameFilters(QStringList()<< QString("%1*").arg(f.completeBaseName()));
-    QFileInfoList list = dir.entryInfoList();
+    if (names.count() > 0){
+        foreach (QString name, names){
+            QFileInfo f(name);
+            QDir dir;
+            if (current_core == "pcsx")
+                dir.setPath(sstate_path);
+            else
+                dir.setPath(rom_path);
+            dir.setFilter(QDir::Files);
+            dir.setSorting(QDir::Name | QDir::IgnoreCase);
+            dir.setNameFilters(QStringList()<< QString("%1*").arg(f.completeBaseName()));
+            QFileInfoList list = dir.entryInfoList();
 
-    res = list.count();
+            res += list.count();
+        }
+    }
     return res;
 }
 
