@@ -29,16 +29,16 @@ public slots:
     void open_help();
     void set_keys();
 
-    void currentRowChanged ( int );
+    void rom_selected ( QListWidgetItem* );
 	void set_rom_dir();
 	void set_bios_dir();
     void process_finished(int);
     void memory_cards();
 
 private:
-    void rom_exec(int slot);
+    void rom_exec(int disc, int slot);
     void get_roms();
-    int count_save_states(QString name);
+    int count_save_states(QStringList name);
 
     Ui::MainWindow *ui;
     QList<RomInfo*> m_Roms;
