@@ -2,6 +2,7 @@
 #include "ui_options.h"
 #include "memorycardselect.h"
 #include "accelerometerconfig.h"
+#include "keys.h"
 
 QStringList current_config=QStringList();
 QString current_core="";
@@ -20,15 +21,6 @@ options::options(QWidget *parent) :
     connect(ui->pbnMemoryCard2, SIGNAL(clicked()),this,SLOT(memory_card_2()));
     connect(ui->pbnConfigAccel, SIGNAL(clicked()), this, SLOT(config_accel()));
 
-    QDir dir( bios_path);
-    dir.setFilter(QDir::Files);
-    dir.setSorting(QDir::Name);
-    dir.setNameFilters(QStringList()<<"*.bin");
-    QFileInfoList list = dir.entryInfoList();
-    for (int i = 0; i < list.size(); ++i) {
-        QFileInfo fileInfo = list.at(i);
-        ui->comboBox_4->addItem(fileInfo.fileName());
-    }
     load_cfg();
 
     m_Loop = NULL;
@@ -45,32 +37,27 @@ void options::hideEvent(QHideEvent *){
     if (!this->isVisible()){
         m_Loop->quit();
 
-        QDialog *dlg=new QDialog(this);
-        dlg->setWindowTitle(tr("Confirm"));
-        QGridLayout *lay=new QGridLayout(dlg);
-        QLabel *lbl=new QLabel(tr("Do you want to save settings?"),dlg);
-        lay->addWidget(lbl,1,0,1,2);
-        QPushButton *yes=new QPushButton(tr("Yes"),dlg);
-        QPushButton *no=new QPushButton(tr("No"),dlg);
-        connect (yes,SIGNAL(clicked()),dlg,SLOT(accept()));
-        connect (no,SIGNAL(clicked()),dlg,SLOT(reject()));
-        connect (dlg,SIGNAL(accepted()),this,SLOT(save_cfg()));
-        connect (dlg,SIGNAL(rejected()),this,SLOT(reload_cfg()));
-        lay->addWidget(yes,2,0);
-        lay->addWidget(no ,2,1);
-        dlg->exec();
+        QMessageBox msg(this);
+        msg.setWindowTitle(tr("Confirm"));
+        msg.setText(tr("Do you want to save settings?"));
+        msg.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
+        if (msg.exec() == QMessageBox::Yes)
+            save_cfg();
+        else
+            reload_cfg();
     }
 }
 
-void options::load_cfg(){
-
+void options::load_cfg()
+{
+    init_data();
     m_MemoryCard1 = "/home/user/.pcsx/memcards/card1.mcd";
     m_MemoryCard2 = "/home/user/.pcsx/memcards/card2.mcd";
 
     QFileInfo f(m_MemoryCard1);
     ui->pbnMemoryCard1->setText(f.fileName());
     f = QFileInfo(m_MemoryCard2);
-    ui->pbnMemoryCard2->setText(f.fileName());
+    ui->pbnMemoryCard2->setText(f.fileName());    
 
     QString path="";
     if (game!="")
@@ -83,8 +70,10 @@ void options::load_cfg(){
     configs << "pcsx-threadedspu" << "pcsx-hwa" << "pcsx-bios" << "pcsx-frameskip";
     configs << "accel" << "cdda" << "xa" << "rcnt" << "sio" << "spuirq" << "vsync" << "fsaa" << "pcsx-fullscreen";
     configs << "pcsx-fps" << "pcsx-cpu" << "pcsx-nofl" << "mcd1" << "mcd2" << "spu_reverb" << "spu_interpolation";
-    configs << "region" << "analog" << "accel_sens" << "accel_y_def" << "accel_max" << "displayon";
+    configs << "region" << "analog" << "accel_sens" << "accel_y_def" << "accel_max" << "displayon" << "keys";
+    configs << "autosave";
 
+    ui->radioButton_2->setChecked(true);
     ui->comboBox_5->setCurrentIndex(0);
     ui->checkBox_9->setChecked(false);
     ui->checkBox_4->setChecked(false);
@@ -94,10 +83,12 @@ void options::load_cfg(){
     ui->cmbInterpolation->setCurrentIndex(0);
     ui->cmbRegion->setCurrentIndex(0);
 
+    ui->cmbKeysConfig->setCurrentIndex(-1);
     ui->chkDisplayOn->setChecked(false);
     ui->numSens->setValue(150);
     ui->numYDef->setValue(500);
     ui->numMaxValue->setValue(500);
+    ui->chkAutosave->setChecked(false);
 
     if (QFile::exists(path)){
         QFile file(path);
@@ -119,15 +110,15 @@ void options::load_cfg(){
                      break;
                  case 2: // bios
                      if (value=="")value="scph1001.bin";
-                     if (ui->comboBox_4->findText(value)!=1)
+                     if (ui->comboBox_4->findText(value) != -1)
                          ui->comboBox_4->setCurrentIndex(ui->comboBox_4->findText(value));
                      break;
                  case 3: // psx4m-cycle
-                     if (ui->comboBox->findText(value)!=1)
+                     if (ui->comboBox->findText(value) != -1)
                          ui->comboBox->setCurrentIndex(ui->comboBox->findText(value));
                      break;
                  case 4: // psx4m-frame
-                     if (ui->comboBox_2->findText(value)!=1)
+                     if (ui->comboBox_2->findText(value) != -1)
                          ui->comboBox_2->setCurrentIndex(ui->comboBox_2->findText(value));
                      break;
                  case 5: // psx4m-hwa
@@ -146,14 +137,14 @@ void options::load_cfg(){
                       if (value=="1")ui->checkBox_3->setChecked(true);else ui->checkBox_3->setChecked(false);
                       break;
                  case 10:// pcsx-hwa
-                     if (ui->comboBox_5->findText(value)!=-1)
+                     if (ui->comboBox_5->findText(value) != -1)
                          ui->comboBox_5->setCurrentIndex(ui->comboBox_5->findText(value));
                       break;
                  case 11:// pcsx-bios
                       if (value=="1")ui->checkBox_5->setChecked(true);else ui->checkBox_5->setChecked(false);                      
                       break;
                  case 12:// pcsx-frameskip
-                      if (ui->comboBox_3->findText(value)!=-1)
+                      if (ui->comboBox_3->findText(value)!= -1)
                           ui->comboBox_3->setCurrentIndex(ui->comboBox_3->findText(value));
                       break;
                  case 13:
@@ -208,11 +199,11 @@ void options::load_cfg(){
                       if (value=="1")ui->chkReverb->setChecked(true);else ui->chkReverb->setChecked(false);
                       break;
                  case 28:
-                     if (ui->cmbInterpolation->findText(value)!=1)
+                     if (ui->cmbInterpolation->findText(value) != -1)
                          ui->cmbInterpolation->setCurrentIndex(ui->cmbInterpolation->findText(value));
                      break;
                  case 29:
-                     if (ui->cmbRegion->findText(value)!=1)
+                     if (ui->cmbRegion->findText(value) != -1)
                          ui->cmbRegion->setCurrentIndex(ui->cmbRegion->findText(value));
                      break;
                  case 30:
@@ -230,6 +221,13 @@ void options::load_cfg(){
                  case 34:
                      if (value=="1")ui->chkDisplayOn->setChecked(true);else ui->chkDisplayOn->setChecked(false);
                      break;
+                 case 35:
+                     if (ui->cmbKeysConfig->findText(value) != -1)
+                         ui->cmbKeysConfig->setCurrentIndex(ui->cmbKeysConfig->findText(value));
+                     break;
+                 case 36:
+                     if (value=="1")ui->chkAutosave->setChecked(true);else ui->chkAutosave->setChecked(false);
+                     break;
                  default:
                       qDebug(QString("Undefined setting:%1 = %2").arg(line.at(0)).arg(value).toAscii());
                       break;
@@ -266,6 +264,12 @@ void options::createCfg(){
     {
         current_core="pcsx";
 
+        if (ui->cmbKeysConfig->currentIndex()>=0){
+            QHash<QString, QString> kList = keys::configList();
+            current_config << "-keys" << kList[ui->cmbKeysConfig->currentText()];
+        }
+
+        if (ui->chkAutosave->isChecked())current_config<<"-autosave";
         if (ui->chkDisplayOn->isChecked())current_config<<"-displayon";
         if (ui->chkAnalog->isChecked())current_config<<"-analog";
         current_config<<"-sens"<<QString::number(ui->numSens->value());
@@ -336,6 +340,7 @@ void options::save_cfg(){
     else
        path=cfg_path+"default";
 
+    qDebug() << "Saving cfg to" << path;
     QString config=QString(
                    "core=%1\n" \
                    "sound=%2\n" \
@@ -371,7 +376,9 @@ void options::save_cfg(){
                    "accel_sens=%32\n" \
                    "accel_y_def=%33\n" \
                    "accel_max=%34\n" \
-                   "displayon=%35\n")
+                   "displayon=%35\n" \
+                   "keys=%36\n" \
+                   "autosave=%37\n")
             .arg(ui->radioButton_2->isChecked()?"1":"0")
             .arg(ui->checkBox->isChecked()?"1":"0")
             .arg(ui->comboBox_4->currentText())
@@ -406,7 +413,9 @@ void options::save_cfg(){
             .arg(ui->numSens->value())
             .arg(ui->numYDef->value())
             .arg(ui->numMaxValue->value())
-            .arg(ui->chkDisplayOn->isChecked()?"1":"0");
+            .arg(ui->chkDisplayOn->isChecked()?"1":"0")
+            .arg(ui->cmbKeysConfig->currentText())
+            .arg(ui->chkAutosave->isChecked()?"1":"0");
 
     QFile file(path);
     QFileInfo pathInfo(path);
@@ -490,12 +499,12 @@ void options::show()
     ui->scrollArea_2->verticalScrollBar()->setValue(0);
     ui->scrollArea_3->verticalScrollBar()->setValue(0);
     ui->scrollArea_4->verticalScrollBar()->setValue(0);
+    ui->scrollArea_5->verticalScrollBar()->setValue(0);
 
     QWidget::show();
 
     m_Loop = new QEventLoop(this);
     m_Loop->exec();
-
 }
 
 void options::accel_toggled(bool toggled)
@@ -518,3 +527,24 @@ void options::config_accel()
 
     delete dlg;
 }
+
+void options::init_data()
+{
+    ui->comboBox_4->clear();
+    QDir dir( bios_path);
+    dir.setFilter(QDir::Files);
+    dir.setSorting(QDir::Name);
+    dir.setNameFilters(QStringList()<<"*.bin");
+    QFileInfoList list = dir.entryInfoList();
+    for (int i = 0; i < list.size(); ++i) {
+        QFileInfo fileInfo = list.at(i);
+        ui->comboBox_4->addItem(fileInfo.fileName());
+    }
+
+    ui->cmbKeysConfig->clear();
+    QHash<QString, QString> kList = keys::configList();
+    QList<QString> keys = kList.keys();
+    qSort(keys.begin(), keys.end());
+    foreach (QString name, keys)
+        ui->cmbKeysConfig->addItem(name);
+}
