@@ -18,6 +18,7 @@ public:
     ~SavestateSelect();
 
     int Selected();
+    int SelectedDisc();
 
 private:
     Ui::SavestateSelect *ui;
@@ -27,9 +28,11 @@ private:
 
     QStringList m_List;
     void FillList();
+    QString GetSavestatePath();
 
 private slots:
     void SavestateSelected(QListWidgetItem*);
+    void DeleteSavestate();
 };
 
 #endif // SAVESTATESELECT_H
